@@ -1,5 +1,5 @@
 /* URL.java -- Uniform Resource Locator Class
-   Copyright (C) 1998, 1999, 2000, 2002, 2003, 2004, 2005
+   Copyright (C) 1998, 1999, 2000, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
@@ -38,6 +38,7 @@ exception statement from your version. */
 
 package java.net;
 
+import gnu.classpath.SystemProperties;
 import gnu.java.net.URLParseError;
 
 import java.io.IOException;
@@ -189,7 +190,8 @@ public final class URL implements Serializable
    * This a table where we cache protocol handlers to avoid the overhead
    * of looking them up each time.
    */
-  private static HashMap ph_cache = new HashMap();
+  private static HashMap<String, URLStreamHandler> ph_cache
+    = new HashMap<String, URLStreamHandler>();
 
   /**
    * Whether or not to cache protocol handlers.
@@ -198,7 +200,7 @@ public final class URL implements Serializable
 
   static
     {
-      String s = System.getProperty("gnu.java.net.nocache_protocol_handlers");
+      String s = SystemProperties.getProperty("gnu.java.net.nocache_protocol_handlers");
 
       if (s == null)
 	cache_handlers = true;
@@ -321,7 +323,8 @@ public final class URL implements Serializable
    */
   public URL(String spec) throws MalformedURLException
   {
-    this((URL) null, spec != null ? spec : "", (URLStreamHandler) null);
+    this((URL) null, spec != null ? spec : "", (URLStreamHandler) null,
+	 false);
   }
 
   /**
@@ -342,7 +345,9 @@ public final class URL implements Serializable
    */
   public URL(URL context, String spec) throws MalformedURLException
   {
-    this(context, spec, (context == null) ? (URLStreamHandler)null : context.ph);
+    this(context, spec,
+	 (context == null) ? (URLStreamHandler) null : context.ph,
+	 false);
   }
 
   /**
@@ -375,6 +380,23 @@ public final class URL implements Serializable
    */
   public URL(URL context, String spec, URLStreamHandler ph)
     throws MalformedURLException
+  {
+    this(context, spec, ph, true);
+  }
+
+  /**
+   * Private constructor called by all other constructors taking
+   * a context and spec.
+   *
+   * @param context The context in which to parse the specification
+   * @param spec The string to parse as an URL
+   * @param ph The stream handler for the URL
+   * @param phFromUser Whether or not the user supplied the URLStreamHandler
+   *
+   */
+  private URL(URL context, String spec, URLStreamHandler ph,
+	      boolean phFromUser)
+    throws MalformedURLException
   {
     /* A protocol is defined by the doc as the substring before a ':'
      * as long as the ':' occurs before any '/'.
@@ -396,49 +418,72 @@ public final class URL implements Serializable
     if ((colon = spec.indexOf("://", 1)) > 0
 	&& ((colon < slash || slash < 0))
         && ! spec.regionMatches(colon, "://:", 0, 4))
-      context = null;
+      {
+	context = null;
+	if (! phFromUser)
+	  ph = null;
+      }
+
+    boolean protocolSpecified = false;
 
     if ((colon = spec.indexOf(':')) > 0
         && (colon < slash || slash < 0))
       {
-	// Protocol specified in spec string.
+	// Protocol may have been specified in spec string.
+        protocolSpecified = true;
 	protocol = spec.substring(0, colon).toLowerCase();
-	if (context != null && context.protocol.equals(protocol))
-	  {
-	    // The 1.2 doc specifically says these are copied to the new URL.
-	    host = context.host;
-	    port = context.port;
-            userInfo = context.userInfo;
-	    authority = context.authority;
-	  }
+	if (context != null)
+          {
+            if (context.protocol.equals(protocol))
+              {
+                // The 1.2 doc specifically says these are copied to the new URL.
+                host = context.host;
+                port = context.port;
+                userInfo = context.userInfo;
+                authority = context.authority;
+              }
+            else
+              {
+                // There was a colon in the spec.  Check to see if
+                // what precedes it is a valid protocol.  If it was
+                // not, assume that it is relative to the context.
+                URLStreamHandler specPh = getURLStreamHandler(protocol.trim());
+                if (null == specPh)
+                    protocolSpecified = false;
+              }
+          }
       }
-    else if (context != null)
+
+    if (!protocolSpecified)
       {
-	// Protocol NOT specified in spec string.
-	// Use context fields (except ref) as a foundation for relative URLs.
-	colon = -1;
-	protocol = context.protocol;
-	host = context.host;
-	port = context.port;
-        userInfo = context.userInfo;
-	if (spec.indexOf(":/", 1) < 0)
-	  {
-	    file = context.file;
-	    if (file == null || file.length() == 0)
-	      file = "/";
-	  }
-	authority = context.authority;
+        if (context != null)
+          {
+            // Protocol NOT specified in spec string.
+            // Use context fields (except ref) as a foundation for relative URLs.
+            colon = -1;
+            protocol = context.protocol;
+            host = context.host;
+            port = context.port;
+            userInfo = context.userInfo;
+            if (spec.indexOf(":/", 1) < 0)
+              {
+                file = context.file;
+                if (file == null || file.length() == 0)
+                  file = "/";
+              }
+            authority = context.authority;
+          }
+        else // Protocol NOT specified in spec. and no context available.
+          throw new MalformedURLException("Absolute URL required with null"
+                                          + " context: " + spec);
       }
-    else // Protocol NOT specified in spec. and no context available.
-      throw new MalformedURLException("Absolute URL required with null"
-				      + " context: " + spec);
 
     protocol = protocol.trim();
 
     if (ph != null)
       {
 	SecurityManager s = System.getSecurityManager();
-	if (s != null)
+	if (s != null && phFromUser)
 	  s.checkPermission(new NetPermission("specifyStreamHandler"));
 
 	this.ph = ph;
@@ -462,7 +507,17 @@ public final class URL implements Serializable
       }
     catch (URLParseError e)
       {
-	throw new MalformedURLException(e.getMessage());
+        MalformedURLException mue = new MalformedURLException(e.getMessage());
+        mue.initCause(e);
+	throw mue;
+      }
+    catch (RuntimeException e)
+      {
+        // This isn't documented, but the JDK also catches
+        // RuntimeExceptions here.
+        MalformedURLException mue = new MalformedURLException(e.getMessage());
+        mue.initCause(e);
+        throw mue;
       }
 
     if (hashAt >= 0)
@@ -515,8 +570,7 @@ public final class URL implements Serializable
    */
   public Object getContent(Class[] classes) throws IOException
   {
-    // FIXME: implement this
-    return getContent();
+    return openConnection().getContent(classes);
   }
 
   /**
@@ -848,7 +902,7 @@ public final class URL implements Serializable
     // First, see if a protocol handler is in our cache.
     if (cache_handlers)
       {
-	if ((ph = (URLStreamHandler) ph_cache.get(protocol)) != null)
+	if ((ph = ph_cache.get(protocol)) != null)
 	  return ph;
       }
 
@@ -867,7 +921,7 @@ public final class URL implements Serializable
 	// Except in very unusual environments the JDK specified one shouldn't
 	// ever be needed (or available).
 	String ph_search_path =
-	  System.getProperty("java.protocol.handler.pkgs");
+	  SystemProperties.getProperty("java.protocol.handler.pkgs");
 
 	// Tack our default package on at the ends.
 	if (ph_search_path != null)
@@ -881,9 +935,9 @@ public final class URL implements Serializable
 	// Cache the systemClassLoader
 	if (systemClassLoader == null)
 	  {
-	    systemClassLoader = (ClassLoader) AccessController.doPrivileged
-	      (new PrivilegedAction() {
-		  public Object run()
+	    systemClassLoader = AccessController.doPrivileged
+	      (new PrivilegedAction<ClassLoader>() {
+		  public ClassLoader run()
 	          {
 		    return ClassLoader.getSystemClassLoader();
 		  }
